@@ -55,6 +55,15 @@
 #endif
 #include <dlfcn.h>
 
+#include <fcntl.h>
+#include <sys/ioctl.h>
+#include <string.h>
+#include <unistd.h>
+#include <asm/ioctl.h>
+#include <asm/types.h>
+#include <asm/arch-omap/omapfb.h>
+#include <sys/socket.h>
+#include <sys/un.h>
 /* INCLUDES imported from adv-backlight */
 #include <gconf/gconf-client.h>
 #include <hildon/hildon-sound.h>
@@ -94,6 +103,7 @@
 #define GCONF_KEY_VOLUME_JUMP GCONF_PREFIX "/volume_jump"
 #define GCONF_KEY_VOLUME_BEEP GCONF_PREFIX "/volume_beep"
 
+#define FBDEV "/dev/fb0"
 
 #define SYSFS_BRIGHTNESS "/sys/devices/platform/omapfb/panel/backlight_level"
 
@@ -187,6 +197,7 @@ static void advanced_backlight_plugin_create_menu (AdvancedBacklightPlugin *);
 static void advanced_backlight_plugin_check_init_rotation (AdvancedBacklightPlugin *);
 static void advanced_backlight_plugin_init_threads (AdvancedBacklightPlugin *);
 static void advanced_backlight_plugin_init_dsme (AdvancedBacklightPlugin *);
+static void advanced_backlight_plugin_update_brightness_value (AdvancedBacklightPluginPrivate *, gint);
 static void advanced_backlight_plugin_dsme_update_brightness_value (AdvancedBacklightPluginPrivate *, gint);
 static void advanced_backlight_plugin_770_update_brightness_value (AdvancedBacklightPluginPrivate *, gint);
 static void advanced_backlight_plugin_update_volume_value (AdvancedBacklightPlugin *, gint);
@@ -218,9 +229,6 @@ static char *keyname[4] = { "Up", "Right", "Down", "Left" };
 static KeyCode keycode[4] = { 111, 114, 116, 113 };
 #endif
 
-int (*dsmesock_connect) ();
-void (*dsmesock_send) (int, struct dsme_message *);
-void (*dsmesock_close) (int);
 
 GtkWidget *
 advanced_backlight_plugin_new (void)
@@ -392,39 +400,10 @@ static void
 advanced_backlight_plugin_init_dsme (AdvancedBacklightPlugin * vsp)
 {
   AdvancedBacklightPluginPrivate *priv = ADVANCED_BACKLIGHT_PLUGIN_GET_PRIVATE (vsp);
-  char *error;
 
   g_debug ("%s\n", __PRETTY_FUNCTION__);
 
-  priv->has_dsme = FALSE;
-  priv->dl_handle = dlopen ("libdsme.so.0.1.0", RTLD_LAZY);
-  if (!priv->dl_handle)
-    {
-      fprintf (stderr, "%s\n", dlerror ());
-      return;
-    }
-  dlerror ();
-
-  *(void **) (&dsmesock_connect) = dlsym (priv->dl_handle, "dsmesock_connect");
-  if ((error = dlerror ()) != NULL)
-    {
-      fprintf (stderr, "%s\n", error);
-      return;
-    }
-
-  *(void **) (&dsmesock_send) = dlsym (priv->dl_handle, "dsmesock_send");
-  if ((error = dlerror ()) != NULL)
-    {
-      fprintf (stderr, "%s\n", error);
-      return;
-    }
-
-  *(void **) (&dsmesock_close) = dlsym (priv->dl_handle, "dsmesock_close");
-  if ((error = dlerror ()) != NULL)
-    {
-      fprintf (stderr, "%s\n", error);
-      return;
-    }
+  /* you probably want to do this in another way -Stskeeps */
   priv->has_dsme = TRUE;
 }
 
@@ -532,14 +511,12 @@ key_pressed (GtkWidget * widget, GdkEventKey * event, AdvancedBacklightPlugin *
 
     case GDK_Left:             /* Brightness down */
       hildon_volumebar_set_level (HILDON_VOLUMEBAR (priv->brightness_slider),
-                                  hildon_volumebar_get_level (HILDON_VOLUMEBAR (priv->brightness_slider)) -
-                                  priv->brightness_jump);
+                                  hildon_volumebar_get_level (HILDON_VOLUMEBAR (priv->brightness_slider)) - priv->brightness_jump);
       return TRUE;
 
     case GDK_Right:            /* Brightness up */
       hildon_volumebar_set_level (HILDON_VOLUMEBAR (priv->brightness_slider),
-                                  hildon_volumebar_get_level (HILDON_VOLUMEBAR (priv->brightness_slider)) +
-                                  priv->brightness_jump);
+                                  hildon_volumebar_get_level (HILDON_VOLUMEBAR (priv->brightness_slider)) + priv->brightness_jump);
       return TRUE;
 
     case GDK_Return:           /* Mute */
@@ -747,7 +724,7 @@ rotation_button_clicked (GtkButton * button, AdvancedBacklightPlugin * vsp)
 static gboolean
 brightness_level_changed (AdvancedBacklightPlugin * vsp)
 {
-  GDK_THREADS_ENTER();
+  GDK_THREADS_ENTER ();
   AdvancedBacklightPluginPrivate *priv = ADVANCED_BACKLIGHT_PLUGIN_GET_PRIVATE (vsp);
   int *value;
 
@@ -758,7 +735,7 @@ brightness_level_changed (AdvancedBacklightPlugin * vsp)
   *value = priv->brightness_level;
   g_debug ("priv->brightness_level=%d\n", priv->brightness_level);
   g_async_queue_push (priv->brightness_queue, value);
-  GDK_THREADS_LEAVE();
+  GDK_THREADS_LEAVE ();
   return FALSE;
 }
 
@@ -774,9 +751,9 @@ brightness_level_changing (GtkWidget * widget, AdvancedBacklightPlugin * vsp)
   gtk_label_set_text (GTK_LABEL (priv->brightness_level_label), (priv->brightness_level_string)->str);
 
   if (priv->timer_id > 0)
-  {
-    g_source_remove(priv->timer_id);
-  }
+    {
+      g_source_remove (priv->timer_id);
+    }
   priv->timer_id = g_timeout_add (BRIGHTNESS_CHANGE_DELAY, (GSourceFunc) brightness_level_changed, vsp);
 }
 
@@ -851,10 +828,10 @@ advanced_backlight_plugin_update_volume (AdvancedBacklightPlugin * vsp)
   priv->mute = hildon_volumebar_get_mute (HILDON_VOLUMEBAR (priv->volume_slider));
   volume = (gint) hildon_volumebar_get_level (HILDON_VOLUMEBAR (priv->volume_slider));
   if (priv->mute)
-  {
-    volume *= -1;
-    g_string_printf (priv->volume_level_string, "%4d-", volume);
-  }
+    {
+      volume *= -1;
+      g_string_printf (priv->volume_level_string, "%4d-", volume);
+    }
   else
     g_string_printf (priv->volume_level_string, "%4d", volume);
 
@@ -1227,7 +1204,7 @@ advanced_backlight_plugin_set_rotation (AdvancedBacklightPlugin * vsp, gint rota
   XRRScreenConfiguration *scr_config;
   Rotation current_rotation;
   gint size;
-  int i;
+  int i, mode, fb;
   Display *dpy;
   gint screen;
 
@@ -1273,6 +1250,16 @@ advanced_backlight_plugin_set_rotation (AdvancedBacklightPlugin * vsp, gint rota
   XRRFreeScreenConfigInfo (scr_config);
   XCloseDisplay (dpy);
 
+  if (priv->is_770)
+    {
+      /* Force automatic update of the LCD when on 770 and rotated */
+      mode = (rotation == 0) ? OMAPFB_MANUAL_UPDATE : OMAPFB_AUTO_UPDATE;
+      if ((fb = open(FBDEV, O_RDWR)) >= 0)
+        {
+          ioctl(fb, OMAPFB_SET_UPDATE_MODE, &mode);
+          close(fb);
+        }
+    }
 }
 
 
@@ -1301,7 +1288,8 @@ static void
 advanced_backlight_plugin_dsme_update_brightness_value (AdvancedBacklightPluginPrivate * priv, gint value)
 {
   struct dsme_message message;
-  gint conn;
+  struct sockaddr_un addr;
+  int dsmesock;
 
   g_debug ("%s\n", __PRETTY_FUNCTION__);
   g_debug ("Setting brightness to %d\n", value);
@@ -1310,13 +1298,26 @@ advanced_backlight_plugin_dsme_update_brightness_value (AdvancedBacklightPluginP
     return;
 
   value = (priv->brightness_level * 2) + 1;
+  dsmesock = socket(PF_UNIX, SOCK_STREAM, 0);
+  /* thanks to Austin Che for DSME code. */
+  if (dsmesock < 0) 
+  	g_error("dsme socket failed");
 
-  conn = dsmesock_connect ();
+  memset(&addr, 0, sizeof(addr));
+  addr.sun_family = AF_UNIX;
+  strcpy(addr.sun_path, "/tmp/dsmesock");
+  
+  if (connect(dsmesock, (struct sockaddr *)&addr, sizeof(addr)) < 0)
+  {
+	close(dsmesock);
+	g_error("couldn't connect to dsme");
+  }
+                                                                
   message.len = 0x0c;
   message.request = 0x289;
   message.value = (void *) value;
-  dsmesock_send (conn, &message);
-  dsmesock_close (conn);
+  send(dsmesock, (void *) &message, sizeof(message), 0);
+  close(dsmesock);
 }
 
 /*
@@ -1373,17 +1374,24 @@ advanced_backlight_plugin_dsme_update_brightness_thread (gpointer d)
       if (prev_value != value)
         {
           g_debug ("priv->brightness_level=%d\n", value);
-
-          if (priv->is_770)
-            advanced_backlight_plugin_770_update_brightness_value (priv, value);
-          else
-            advanced_backlight_plugin_dsme_update_brightness_value (priv, value);
+          advanced_backlight_plugin_update_brightness_value (priv, value);
           prev_value = value;
         }
     }
   return g_thread_self ();
 }
 
+static void
+advanced_backlight_plugin_update_brightness_value (AdvancedBacklightPluginPrivate * priv, gint value)
+{
+  if (priv->is_770)
+    advanced_backlight_plugin_770_update_brightness_value (priv, value);
+  else
+    advanced_backlight_plugin_dsme_update_brightness_value (priv, value);
+
+  gconf_client_set_int (priv->gconf_client, GCONF_KEY_BRIGHTNESS, value, NULL);
+}
+
 static void
 advanced_backlight_plugin_770_update_brightness_value (AdvancedBacklightPluginPrivate * priv, gint value)
 {
@@ -1395,11 +1403,11 @@ advanced_backlight_plugin_770_update_brightness_value (AdvancedBacklightPluginPr
   myfile = fopen (SYSFS_BRIGHTNESS, "w");
 
   if (myfile != NULL)
-  {
-    g_debug ("Setting 770 brightness to %d\n", value);
-    fprintf (myfile, "%d\n", value);
-    fclose (myfile);
-  }
+    {
+      g_debug ("Setting 770 brightness to %d\n", value);
+      fprintf (myfile, "%d\n", value);
+      fclose (myfile);
+    }
 }
 
 /*
@@ -1575,14 +1583,12 @@ advanced_backlight_plugin_configure_applet (AdvancedBacklightPlugin * vsp)
   allow_zero_checkbox = gtk_check_button_new ();
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (allow_zero_checkbox), priv->allow_zero);
   allow_zero_caption =
-    hildon_caption_new (NULL, _("Allow zero backlight\n(N800 users beware!)"), allow_zero_checkbox, NULL,
-                        HILDON_CAPTION_MANDATORY);
+    hildon_caption_new (NULL, _("Allow zero backlight\n(N800 users beware!)"), allow_zero_checkbox, NULL, HILDON_CAPTION_MANDATORY);
   hildon_caption_set_size_group (HILDON_CAPTION (allow_zero_caption), GTK_SIZE_GROUP (size_group));
 
   volume_beep_checkbox = gtk_check_button_new ();
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (volume_beep_checkbox), priv->volume_beep);
-  volume_beep_caption =
-    hildon_caption_new (NULL, _("Beep on volume change"), volume_beep_checkbox, NULL, HILDON_CAPTION_MANDATORY);
+  volume_beep_caption = hildon_caption_new (NULL, _("Beep on volume change"), volume_beep_checkbox, NULL, HILDON_CAPTION_MANDATORY);
   hildon_caption_set_size_group (HILDON_CAPTION (volume_beep_caption), GTK_SIZE_GROUP (size_group));
 
   choose_theme = gtk_combo_box_new_text ();
@@ -1707,26 +1713,26 @@ advanced_backlight_plugin_load_icons (AdvancedBacklightPlugin * vsp)
       for (j = 0; j < NUM_VOLUME_ICONS; j++)
         {
           for (m = 0; m < 2; m++)
-	    {
-            if (m == 0)
-              snprintf (icon_name, 30, "adv-backlight-icon-%d.%d.u.png", i, j);
-	    else
-              snprintf (icon_name, 30, "adv-backlight-icon-%d.%d.m.png", i, j);
-  
-            file_name = g_build_filename (ICON_DIR, priv->current_theme, icon_name, NULL);
-            if (priv->pixbuf[i][j][m] != NULL)
-              {
-                gdk_pixbuf_unref (priv->pixbuf[i][j][m]);
-              }
-            err = NULL;
-            if (!(priv->pixbuf[i][j][m] = gdk_pixbuf_new_from_file (file_name, &err)))
-              {
-                g_critical ("Unable to read file: %s\n", err->message);
-                g_error_free (err);
-              }
-  
-            g_free (file_name);
-	    }
+            {
+              if (m == 0)
+                snprintf (icon_name, 30, "adv-backlight-icon-%d.%d.u.png", i, j);
+              else
+                snprintf (icon_name, 30, "adv-backlight-icon-%d.%d.m.png", i, j);
+
+              file_name = g_build_filename (ICON_DIR, priv->current_theme, icon_name, NULL);
+              if (priv->pixbuf[i][j][m] != NULL)
+                {
+                  gdk_pixbuf_unref (priv->pixbuf[i][j][m]);
+                }
+              err = NULL;
+              if (!(priv->pixbuf[i][j][m] = gdk_pixbuf_new_from_file (file_name, &err)))
+                {
+                  g_critical ("Unable to read file: %s\n", err->message);
+                  g_error_free (err);
+                }
+
+              g_free (file_name);
+            }
         }
     }
 }
@@ -1739,7 +1745,7 @@ advanced_backlight_plugin_update_icons (AdvancedBacklightPlugin * vsp)
 
   g_debug ("%s\n", __PRETTY_FUNCTION__);
 
-  volume = ABS(priv->volume_level);
+  volume = ABS (priv->volume_level);
   volume_icon = volume / (100 / (NUM_VOLUME_ICONS - 1) + 1);
   if (volume_icon > NUM_VOLUME_ICONS - 1)
     volume_icon = NUM_VOLUME_ICONS - 1;
